module.exports = function (grunt) {
	var moduleName = 'opl';
    var backendModules = ['../backend/src/Teknisa'];
	var directories = ['utilities', 'repositories', 'services', 'controllers'];
	var subModules = ['consolidacao', 'cadastros', 'parametros', 'relatorios', 'documentosFiscais',
		
		// SubModulo Consolidacao
		'consolidacao/assets/modules/autoforne',
		'consolidacao/assets/modules/solicitacao'
	];
	var libs = [
		"./bower_components/zh-preferences/dist/preferences.js",
		"./bower_components/zh-filter-preferences/dist/filter-preferences.js",
        "./bower_components/zh-layout-preferences/dist/layout-preferences.js",
		"./bower_components/zh-grid-configuration/dist/zh-grid-configuration.js",
        "./bower_components/zh-view-preferences/dist/view-preferences.js",
		"./bower_components/zh-reports/dist/zhReports.js",
		"./bower_components/zh-rich-text/dist/zh-rich-text.js",
		"./bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js",
		"./bower_components/zh-interval-component/dist/zh-interval-component.js",
		"./bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js",
		"./bower_components/zh-cache-bypass/src/CacheBypass.js",
		"./bower_components/zh-charts/dist/zh-charts.js",
		"./bower_components/zhuaclfrontend/assets/dist/main.js",
		"./bower_components/zh-image-uploader/dist/zh-image-uploader.js",
		"./bower_components/zhulog/dist/main.js",
		"./bower_components/tek-libraries/dist/main.js",
		"./assets/js/libraries/QRCodeGenerator.js"
	];

	grunt.initConfig({
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPaths(),
				dest: './assets/js/app.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPaths(),
				dest: './assets/css/app.css'
			},
			libs: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: libs,
				dest: 'assets/js/libs.js'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'./assets/js/app.min.js': getJSPaths()
				}
			}
		},
		jshint: {
			all: getJSPaths().concat(["GruntFile.js"]),
			options: {
				evil: true,
				esversion: 6
			}
		},
		watch: {
			js: {
				files: getJSPaths().concat(["GruntFile.js"]),
				tasks: ['jshint', 'concat:js']
			},
			css: {
				files: getCSSPaths(),
				tasks: ['concat:css']
			}
		},
		cssmin: {
			dist: {
				src: getCSSPaths(),
				dest: './assets/css/app.min.css'
			}
		},
		// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
		zhIdGenerator: {
			options: {
				jsonPath: getJSONPaths(),
				menuPath: ["./assets/json/menu.json"]
			}
		},
		buildAclStruct: {
			options: {
				menuConfigFile: "./menuConfig.json"
			}
		},
		buildEnvironment: {
			development: {
				configFile: "./config/devEnvironment.json"
			},
			production: {
				configFile: "./config/prodEnvironment.json"
			},
			test: {
				configFile: "./config/testEnvironment.json"
			},
			srcFolders: ["../backend/config/", "./config", "./", "./assets", "./assets/json", "../backend/src/teknisa/opl/reports"]
		},
		zhWords: {
			options: {
				serverUrl: "https://translate.teknisa.com/backend/index.php",
				organization: "20",
				project: "42689"
			}
		}
	});

	function getJSPaths() {
		var paths = [];
		directories.forEach(function (directory) {
			paths.push('./assets/js/' + directory + '/**/*.js');
			subModules.forEach(function (subModule) {
				paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
			});
		});
		return paths;
	}

	function getCSSPaths() {
		var paths = [];
		paths.push('./assets/css/**/*.css');
		paths.push('!./assets/css/app.css');
		subModules.forEach(function (subModule) {
			paths.push('./assets/modules/' + subModule + '/assets/css/**/*.css');
			paths.push('!./assets/modules/' + subModule + '/assets/css/app.css');
		});
		return paths;
	}

	function getJSONPaths() {
		var paths = ["./assets/json"];
		subModules.forEach(function (subModule) {
			paths.push("./assets/modules/" + subModule + "/assets/json");
		});
		return paths;
	}
	
	function unicodeEscape(s) {
      return s.replace(/[^0-9|^a-z]/gi, (char) => { 
        return `\\u${(`000${char.charCodeAt(0).toString(16).toUpperCase()}`).substr(-4)}`;
      });
    }

	function buildFileContent(srcFiles, filePath, directory) {
		var parts = filePath.split('/');
		
		parts.shift();
		parts.shift();
		parts.shift();
        parts.shift();
        
		var filename = parts[parts.length - 1];
		var extension = filename.split('.').pop().toUpperCase();
		
		if (extension == 'JSON' || extension == 'XML') {
			if (directory == 'datasources') {
				filename = parts.pop();
				filename = filename.replace(/\.[^/.]+$/, "");
				srcFiles[directory][filename] = './../src/Teknisa/' + parts.join('/');
			} else {
				srcFiles[directory].push('/src/Teknisa/' + parts.join('/'));
			}
		}
	}
	
	grunt.registerTask('reportTranslate', 'build translate to BIRT reports', function () {
		var languages = ['en_us', 'es_cl', 'es_co', 'es_es'];
		var srcWords = "assets/json/words/";
		var srcMessages = "assets/json/words/";
		var srcReport = "../../teknisa/est/reports/translationfiles/";
		var basePTBR = grunt.file.readJSON(srcMessages+"en_us.json");
        var languagePTBR = [];
        
        // regex to format properties file content
		var regexEmpty = /[\/\{\}\[\]\"]*/gm;
		var regexEqual = /\:/gm;
		var regexEnter = /\,/gm;
		var regexDoubleScape = /\\\\/gm;
		var regexScapedSpace = /u0020/gm;
		
        
        //Cria arquivo de tradução pt_br
        Object.keys(basePTBR).forEach( key => {
            languagePTBR.push({[unicodeEscape(key)]: unicodeEscape(key)});
        });
        
		var stringPTBRtoSave = JSON.stringify(languagePTBR).replace(regexEqual,'=').replace(regexEmpty,'').replace(regexEnter,'\n').replace(regexDoubleScape,'\\').replace(regexScapedSpace, ' ');
		grunt.file.write(srcReport+'translation'+".properties", stringPTBRtoSave);
		
		languages.forEach( lang => {
			var words = grunt.file.readJSON(srcWords+lang+".json");
			var properties = [];
			var filename = '';

			Object.keys(words).forEach(key => {
			    try{
				    properties.push({[unicodeEscape(key)]: unicodeEscape(words[key])});
			    }catch(err){
			        console.log(key);
			        throw err;
			    }
			});
			
			var stringToSave = JSON.stringify(properties).replace(regexEqual,'=').replace(regexEmpty,'').replace(regexEnter,'\n').replace(regexDoubleScape, '\\' ).replace(regexScapedSpace, ' ');
			
			filename = lang.split('_'); 
			filename = filename.shift()+'_'+filename.shift().toUpperCase(); 
			grunt.file.write(srcReport+'translation_'+filename+".properties", stringToSave); 
			grunt.file.write(srcReport+'translation_'+lang+".properties", stringToSave);
		});
	});

	grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var directories = ['config', 'datasources'];
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		
		directories.forEach(function (directory) {
			backendModules.forEach(function (module) {
				var moduleParts = module.split('/');
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/' + moduleName.toUpperCase() + '/Modules/*');

				subModules.forEach(function (subModule) {
					grunt.file.expand({
						filter: 'isFile'
					}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
						buildFileContent(srcFiles, filePath, directory);
					});
				});

				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + moduleName.toUpperCase() + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	});

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-words');

	grunt.registerTask('default', ['zhWords','jshint', 'backendStep', 'concat', 'buildAclStruct', 'buildEnvironment']);
	grunt.registerTask('mergeBranch', ['zhIdGenerator', 'buildAclStruct']);
	grunt.registerTask('final', ['cssmin', 'uglify']);
};